@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # @Date    : 2020/7/6
 # @Author  : mingming.xu
-# @Email   : mingming.xu@zhaopin.com
+# @Email   : xv44586@gmail.com
 # @File    : optimizers.py
 import re
 
@@ -92,6 +92,177 @@ class Adam(keras.optimizers.Optimizer):
         return dict(list(basic_config.items()) + list(config.items()))
 
 
+class AdaBelief(keras.optimizers.Optimizer):
+    """AdaBelief optimizer.
+
+    Default parameters follow those provided in the original paper.
+
+    # Arguments
+        learning_rate: float >= 0. Learning rate.
+        beta_1: float, 0 < beta < 1. Generally close to 1.
+        beta_2: float, 0 < beta < 1. Generally close to 1.
+        amsgrad: boolean. Whether to apply the AMSGrad variant of this
+            algorithm from the paper "On the Convergence of Adam and
+            Beyond".
+
+    # References
+        - [Adam - A Method for Stochastic Optimization](
+           https://arxiv.org/abs/1412.6980v8)
+        - [AdaBelief Optimizer: Adapting Stepsizes by the Belief in Observed Gradients](
+           https://arxiv.org/pdf/2010.07468.pdf)
+    """
+
+    def __init__(self, learning_rate=0.001, beta_1=0.9, beta_2=0.999,
+                 amsgrad=False, **kwargs):
+        self.initial_decay = kwargs.pop('decay', 0.0)
+        self.epsilon = kwargs.pop('epsilon', K.epsilon())
+        learning_rate = kwargs.pop('lr', learning_rate)
+        super(AdaBelief, self).__init__(**kwargs)
+        with K.name_scope(self.__class__.__name__):
+            self.iterations = K.variable(0, dtype='int64', name='iterations')
+            self.learning_rate = K.variable(learning_rate, name='learning_rate')
+            self.beta_1 = K.variable(beta_1, name='beta_1')
+            self.beta_2 = K.variable(beta_2, name='beta_2')
+            self.decay = K.variable(self.initial_decay, name='decay')
+        self.amsgrad = amsgrad
+
+    @K.symbolic
+    def get_updates(self, loss, params):
+        grads = self.get_gradients(loss, params)
+        self.updates = [K.update_add(self.iterations, 1)]
+
+        lr = self.learning_rate
+        if self.initial_decay > 0:
+            lr = lr * (1. / (1. + self.decay * K.cast(self.iterations,
+                                                      K.dtype(self.decay))))
+
+        t = K.cast(self.iterations, K.floatx()) + 1
+        lr_t = lr * (K.sqrt(1. - K.pow(self.beta_2, t)) /
+                     (1. - K.pow(self.beta_1, t)))
+
+        ms = [K.zeros(K.int_shape(p),
+              dtype=K.dtype(p),
+              name='m_' + str(i))
+              for (i, p) in enumerate(params)]
+        vs = [K.zeros(K.int_shape(p),
+              dtype=K.dtype(p),
+              name='v_' + str(i))
+              for (i, p) in enumerate(params)]
+
+        if self.amsgrad:
+            vhats = [K.zeros(K.int_shape(p),
+                     dtype=K.dtype(p),
+                     name='vhat_' + str(i))
+                     for (i, p) in enumerate(params)]
+        else:
+            vhats = [K.zeros(1, name='vhat_' + str(i))
+                     for i in range(len(params))]
+        self.weights = [self.iterations] + ms + vs + vhats
+
+        for p, g, m, v, vhat in zip(params, grads, ms, vs, vhats):
+            m_t = (self.beta_1 * m) + (1. - self.beta_1) * g
+            v_t = (self.beta_2 * v) + (1. - self.beta_2) * K.square(g - m_t)
+            if self.amsgrad:
+                vhat_t = K.maximum(vhat, v_t)
+                p_t = p - lr_t * m_t / (K.sqrt(vhat_t) + self.epsilon)
+                self.updates.append(K.update(vhat, vhat_t))
+            else:
+                p_t = p - lr_t * m_t / (K.sqrt(v_t) + self.epsilon)
+
+            self.updates.append(K.update(m, m_t))
+            self.updates.append(K.update(v, v_t))
+            new_p = p_t
+
+            # Apply constraints.
+            if getattr(p, 'constraint', None) is not None:
+                new_p = p.constraint(new_p)
+
+            self.updates.append(K.update(p, new_p))
+        return self.updates
+
+    def get_config(self):
+        config = {'learning_rate': float(K.get_value(self.learning_rate)),
+                  'beta_1': float(K.get_value(self.beta_1)),
+                  'beta_2': float(K.get_value(self.beta_2)),
+                  'decay': float(K.get_value(self.decay)),
+                  'epsilon': self.epsilon,
+                  'amsgrad': self.amsgrad}
+        base_config = super(AdaBelief, self).get_config()
+        return dict(list(base_config.items()) + list(config.items()))
+
+
+class AdaBeliefTf(keras.optimizers.Optimizer):
+    """tf.keras 版
+    """
+
+    def __init__(
+            self,
+            learning_rate=0.001,
+            beta_1=0.9,
+            beta_2=0.999,
+            epsilon=1e-6,
+            bias_correct=True,
+            **kwargs
+    ):
+        kwargs['name'] = kwargs.get('name') or 'AdaBelief'
+        super(AdaBeliefTf, self).__init__(**kwargs)
+        self._set_hyper('learning_rate', learning_rate)
+        self._set_hyper('beta_1', beta_1)
+        self._set_hyper('beta_2', beta_2)
+        self.epsilon = epsilon or K.epislon()
+        self.bias_correct = bias_correct
+
+    def _create_slots(self, var_list):
+        for var in var_list:
+            self.add_slot(var, 'm')
+            self.add_slot(var, 'v')
+
+    def _resource_apply(self, grad, var, indices=None):
+        # 准备变量
+        var_dtype = var.dtype.base_dtype
+        lr_t = self._decayed_lr(var_dtype)
+        m = self.get_slot(var, 'm')
+        v = self.get_slot(var, 'v')
+        beta_1_t = self._get_hyper('beta_1', var_dtype)
+        beta_2_t = self._get_hyper('beta_2', var_dtype)
+        epsilon_t = K.cast(self.epsilon, var_dtype)
+        local_step = K.cast(self.iterations + 1, var_dtype)
+        beta_1_t_power = K.pow(beta_1_t, local_step)
+        beta_2_t_power = K.pow(beta_2_t, local_step)
+
+        # 更新公式
+
+        m_t = K.update(m, beta_1_t * m + (1 - beta_1_t) * grad)
+        v_t = K.update(v, beta_2_t * v + (1 - beta_2_t) * (grad - m_t) ** 2)
+
+        # 返回算子
+        with tf.control_dependencies([m_t, v_t]):
+            if self.bias_correct:
+                m_t = m_t / (1.0 - beta_1_t_power)
+                v_t = v_t / (1.0 - beta_2_t_power)
+            var_t = var - lr_t * m_t / (K.sqrt(v_t) + epsilon_t)
+            return K.update(var, var_t)
+
+    def _resource_apply_dense(self, grad, var):
+        return self._resource_apply(grad, var)
+
+    def _resource_apply_sparse(self, grad, var, indices):
+        grad = tf.IndexedSlices(grad, indices, K.shape(var))
+        grad = tf.convert_to_tensor(grad)
+        return self._resource_apply_dense(grad, var)
+
+    def get_config(self):
+        config = {
+            'learning_rate': self._serialize_hyperparameter('learning_rate'),
+            'beta_1': self._serialize_hyperparameter('beta_1'),
+            'beta_2': self._serialize_hyperparameter('beta_2'),
+            'epsilon': self.epsilon,
+            'bias_correct': self.bias_correct,
+        }
+        base_config = super(AdaBeliefTf, self).get_config()
+        return dict(list(base_config.items()) + list(config.items()))
+
+
 def export_to_custom_objects(extend_with_func):
     def new_extend_with_func(BaseOptimizer, name=None):
         NewOptimizer = extend_with_func(BaseOptimizer)
@@ -381,11 +552,13 @@ if is_tf_keras:
     extend_with_piecewise_linear_lr = extend_with_piecewise_linear_lr_tf2
     extend_with_gradient_accumulation = extend_with_gradient_accumulation_tf2
     extend_with_weight_decay = extend_with_weight_decay_tf2
+    AdaBelief = AdaBeliefTf
 else:
     Adam = keras.optimizers.Adam
 
 custom_objects = {
-    'Adam': Adam
+    'Adam': Adam,
+    'AdaBelief': AdaBelief,
 }
 
 keras.utils.get_custom_objects().update(custom_objects)
